@@ -10,14 +10,14 @@ sub import {
     my ($class, %params) = @_;
 
     # set default context
-    IO::Socket::SSL::set_ctx_defaults(ca_file => $params{'ca_cert_file'})
-        if $params{'ca_cert_file'};
-    IO::Socket::SSL::set_ctx_defaults(ca_path => $params{'ca_cert_dir'})
-        if $params{'ca_cert_dir'};
+    IO::Socket::SSL::set_ctx_defaults(ca_file => $params{ca_cert_file})
+        if $params{ca_cert_file};
+    IO::Socket::SSL::set_ctx_defaults(ca_path => $params{ca_cert_dir})
+        if $params{ca_cert_dir};
 }
 
 sub _extra_sock_opts {
-    my ($self, $host, $port) = @_;
+    my ($self, $host) = @_;
 
     return (
         SSL_verify_mode     => $self->{ua}->{ssl_check} ?
@@ -27,6 +27,7 @@ sub _extra_sock_opts {
     );
 }
 
+## no critic (ProhibitMultiplePackages)
 package FusionInventory::Agent::HTTP::Protocol::https::Socket;
 
 use base qw(Net::HTTPS LWP::Protocol::http::SocketMethods);
