@@ -17,12 +17,13 @@ our @EXPORT = qw(
     getDirectoryHandle
     getFileHandle
     getFormatedLocalTime
-    getFormatedGmTime
+    getFormatedGMTTime
     getFormatedDate
     getCanonicalManufacturer
     getCanonicalSpeed
     getCanonicalSize
     getSanitizedString
+    trimWhitespace
     getFirstLine
     getFirstMatch
     getLastLine
@@ -43,6 +44,7 @@ our @EXPORT = qw(
     module2file
     runFunction
     delay
+    slurp
 );
 
 my $nowhere = $OSNAME eq 'MSWin32' ? 'nul' : '/dev/null';
@@ -57,6 +59,8 @@ if ($OSNAME ne 'MSWin32') {
 sub getFormatedLocalTime {
     my ($time) = @_;
 
+    return unless $time;
+
     my ($year, $month , $day, $hour, $min, $sec) =
         (localtime ($time))[5, 4, 3, 2, 1, 0];
 
@@ -65,7 +69,7 @@ sub getFormatedLocalTime {
     );
 }
 
-sub getFormatedGmTime {
+sub getFormatedGMTTime {
     my ($time) = @_;
 
     my ($year, $month , $day, $hour, $min, $sec) =
@@ -105,13 +109,13 @@ sub getCanonicalManufacturer {
         pioneer
     )/xi) {
         $manufacturer = ucfirst(lc($1));
-    } elsif ($manufacturer =~ /^(hp|HP|hewlett packard)/) {
-        $manufacturer = "Hewlett Packard";
-    } elsif ($manufacturer =~ /^(WDC|[Ww]estern)/) {
+    } elsif ($manufacturer =~ /^(hp|HP|(?i)hewlett[ -]packard)/) {
+        $manufacturer = "Hewlett-Packard";
+    } elsif ($manufacturer =~ /^(WDC|(?i)western)/) {
         $manufacturer = "Western Digital";
-    } elsif ($manufacturer =~ /^(ST|[Ss]eagate)/) {
+    } elsif ($manufacturer =~ /^(ST|(?i)seagate)/) {
         $manufacturer = "Seagate";
-    } elsif ($manufacturer =~ /^(HD|IC|HU)/) {
+    } elsif ($manufacturer =~ /^(HD|IC|HU|HGST)/) {
         $manufacturer = "Hitachi";
     }
 
@@ -127,7 +131,7 @@ sub getCanonicalSpeed {
 
     return 400 if $speed =~ /^PC3200U/;
 
-    return undef unless $speed =~ /^([\,\.\d]+) \s? (\S+)$/x;
+    return undef unless $speed =~ /^([,.\d]+) \s? (\S+)$/x;
     my $value = $1;
     my $unit = lc($2);
 
@@ -138,7 +142,8 @@ sub getCanonicalSpeed {
 }
 
 sub getCanonicalSize {
-    my ($size) = @_;
+    my ($size, $base) = @_;
+    $base ||= 1000;
 
     ## no critic (ExplicitReturnUndef)
 
@@ -146,16 +151,20 @@ sub getCanonicalSize {
 
     return $size if $size =~ /^\d+$/;
 
-    return undef unless $size =~ /^(\d+) \s (\S+)$/x;
+    $size =~ s/ //g;
+
+    return undef unless $size =~ /^([,.\d]+) (\S+)$/x;
     my $value = $1;
     my $unit = lc($2);
 
+    use integer;
     return
-        $unit eq 'tb' ? $value * 1000 * 1000 :
-        $unit eq 'gb' ? $value * 1000        :
-        $unit eq 'mb' ? $value               :
-        $unit eq 'kb' ? $value * 0.001       :
-                        undef                ;
+        $unit eq 'tb'    ? $value * $base * $base   :
+        $unit eq 'gb'    ? $value * $base           :
+        $unit eq 'mb'    ? $value                   :
+        $unit eq 'kb'    ? $value / ($base)         :
+        $unit eq 'bytes' ? $value / ($base * $base) :
+                           undef                    ;
 }
 
 sub compareVersion {
@@ -185,7 +194,7 @@ sub getSanitizedString {
     $string =~ s/[[:cntrl:]]//g;
 
     # encode to utf-8 if needed
-    if ($string !~ m/\A(
+    if (!Encode::is_utf8($string) && $string !~ m/\A(
           [\x09\x0A\x0D\x20-\x7E]           # ASCII
         | [\xC2-\xDF][\x80-\xBF]            # non-overlong 2-byte
         | \xE0[\xA0-\xBF][\x80-\xBF]        # excluding overlongs
@@ -201,8 +210,16 @@ sub getSanitizedString {
     return $string;
 }
 
+sub trimWhitespace {
+    my ($value) = @_;
+    $value =~ s/^\s+//;
+    $value =~ s/\s+$//;
+    $value =~ s/\s+/ /g;
+    return $value;
+}
+
 sub getDirectoryHandle {
-    my %params = @_;
+    my (%params) = @_;
 
     die "no directory parameter given" unless $params{directory};
 
@@ -218,7 +235,7 @@ sub getDirectoryHandle {
 }
 
 sub getFileHandle {
-    my %params = @_;
+    my (%params) = @_;
 
     my $handle;
 
@@ -233,6 +250,9 @@ sub getFileHandle {
             last SWITCH;
         }
         if ($params{command}) {
+            # Turn off localised output for commands
+            local $ENV{LC_ALL} = 'C';
+            local $ENV{LANG} = 'C';
             if (!open $handle, '-|', $params{command} . " 2>$nowhere") {
                 $params{logger}->error(
                     "Can't run command $params{command}: $ERRNO"
@@ -241,10 +261,10 @@ sub getFileHandle {
             }
             last SWITCH;
         }
-	if ($params{string}) {
-	    open $handle, "<", \$params{string} or die;
+        if ($params{string}) {
+            open $handle, "<", \$params{string} or die;
             last SWITCH;
-	}
+        }
         die "neither command, file or string parameter given";
     }
 
@@ -252,7 +272,7 @@ sub getFileHandle {
 }
 
 sub getFirstLine {
-    my %params = @_;
+    my (%params) = @_;
 
     my $handle = getFileHandle(%params);
     return unless $handle;
@@ -265,7 +285,7 @@ sub getFirstLine {
 }
 
 sub getLastLine {
-    my %params = @_;
+    my (%params) = @_;
 
     my $handle = getFileHandle(%params);
     return unless $handle;
@@ -281,7 +301,7 @@ sub getLastLine {
 }
 
 sub getFirstMatch {
-    my %params = @_;
+    my (%params) = @_;
 
     return unless $params{pattern};
     my $handle = getFileHandle(%params);
@@ -298,7 +318,7 @@ sub getFirstMatch {
 }
 
 sub getAllLines {
-    my %params = @_;
+    my (%params) = @_;
 
     my $handle = getFileHandle(%params);
     return unless $handle;
@@ -316,7 +336,7 @@ sub getAllLines {
 }
 
 sub getLinesCount {
-    my %params = @_;
+    my (%params) = @_;
 
     my $handle = getFileHandle(%params);
     return unless $handle;
@@ -333,7 +353,9 @@ sub getLinesCount {
 sub canRun {
     my ($binary) = @_;
 
-    return scalar(which($binary));
+    return $binary =~ m{^/} ?
+        -x $binary :            # full path
+        scalar(which($binary)); # executable name
 }
 
 sub canRead {
@@ -355,7 +377,8 @@ sub hex2char {
     return undef unless $value;
     return $value unless $value =~ /^0x/;
 
-    $value =~ s/^0x//;
+    $value =~ s/^0x//; # drop hex prefix
+    $value =~ s/00$//; # drop trailing null-character
     $value =~ s/(\w{2})/chr(hex($1))/eg;
 
     return $value;
@@ -442,15 +465,16 @@ sub runFunction {
 
     my $result;
     eval {
-        # set 
         local $SIG{ALRM} = sub { die "alarm\n" };
+        # set a timeout if needed
         alarm $params{timeout} if $params{timeout};
-        no strict 'refs'; ## no critic
+
+        no strict 'refs'; ## no critic (ProhibitNoStrict)
         $result = &{$params{module} . '::' . $params{function}}(
             ref $params{params} eq 'HASH'  ? %{$params{params}} :
             ref $params{params} eq 'ARRAY' ? @{$params{params}} :
-                                               $params{params} 
-	);
+                                               $params{params}
+        );
         alarm 0;
     };
 
@@ -475,17 +499,27 @@ sub delay {
     }
 }
 
+sub slurp {
+    my($file) = @_;
+
+    my $handler;
+    return unless open $handler, '<', $file;
+    local $INPUT_RECORD_SEPARATOR; # Set input to "slurp" mode.
+    my $content = <$handler>;
+    close $handler;
+    return $content;
+}
 
 1;
 __END__
 
 =head1 NAME
 
-FusionInventory::Agent::Tools - OS-independant generic functions
+FusionInventory::Agent::Tools - OS-independent generic functions
 
 =head1 DESCRIPTION
 
-This module provides some OS-independant generic functions.
+This module provides some OS-independent generic functions.
 
 =head1 FUNCTIONS
 
@@ -493,7 +527,7 @@ This module provides some OS-independant generic functions.
 
 Returns a formated date from given Unix timestamp.
 
-=head2 getFormatedGmTime($time)
+=head2 getFormatedGMTTime($time)
 
 Returns a formated date from given Unix timestamp.
 
@@ -509,7 +543,7 @@ Returns a normalized manufacturer value for given one.
 
 Returns a normalized speed value (in Mhz) for given one.
 
-=head2 getCanonicalSize($size)
+=head2 getCanonicalSize($size, $base)
 
 Returns a normalized size value (in Mb) for given one.
 
@@ -518,6 +552,11 @@ Returns a normalized size value (in Mb) for given one.
 Returns the input stripped from any control character, properly encoded in
 UTF-8.
 
+=head2 trimWhitespace($string)
+
+Remove leading and trailing whitespace, and fold multiple whitespace
+characters into a single one.
+
 =head2 compareVersion($major, $minor, $min_major, $min_minor)
 
 Returns true if software with given major and minor version meet minimal
@@ -568,8 +607,9 @@ of line removed.
 
 =head2 getAllLines(%params)
 
-Returns all the lines of given command output or given file content, with end
-of line removed.
+Returns all the lines of given command output or given file content, as a list
+of strings with end of line removed in list context, as a single string
+otherwise.
 
 =over
 
@@ -702,3 +742,7 @@ Run a function whose name is computed at runtime and return its result.
 
 Wait for $second. It uses sleep() or Win32::Sleep() depending
 on the Operating System.
+
+=head2 slurp($file)
+
+Return the content of a given file.
